# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-11-27 01:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reporting', '0006_reportdef_calc_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='reportdef',
            name='calc_type',
            field=models.CharField(choices=[('as_of', 'Point in Time'), ('diff', 'Change Over Time')], max_length=20),
        ),
    ]
