#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='accountifie',
      description='Core Accouting Functionality',
      author='Ian McAllister',
      version='0.3.4',
      author_email='ian@mcallisternevins.com',
      url='https://github.com/imcallister/accountifie',
      packages=find_packages(),
      include_package_data=True,
      )
